#include "globals.h"

int km::LoadPointsFromFile(const QString& filename,
                           int window_begin, int window_end, int samples_begin, int samples_end,
                           int& num_records, int& num_samples, Scalar*& records)
{
    FILE* in = fopen(filename.toAscii(), "rb");
    if (!in)
    {
        cerr << "File name " << filename.toStdString() << " not valid, no clustering done.";
        fclose(in);
        return -1;
    }

    WfmHeader header;
    fread(&header, sizeof(WfmHeader), 1, in);

    int num_all_samples = header.num_samples;
    num_records = window_end - window_begin;
    num_samples = samples_end - samples_begin;
    records = (Scalar*)malloc(num_records * num_samples * sizeof(Scalar));
    if (records == NULL)
    {
        cerr << "File " << filename.toStdString() << " could not be read, possibly due to " \
                "insufficient memory (malloc failed)." << endl;
        fclose(in);
        return -1;
    }
    int bytes_per_sample = header.bps,
            bytes_per_record = bytes_per_sample * num_samples;

    fseek(in, window_begin * bytes_per_record, SEEK_CUR);
    for (int i=0; i<num_records; ++i)
    {
        fseek(in, samples_begin*bytes_per_sample, SEEK_CUR);
        for (int j=0; j<num_samples; ++j)
        {
            double val;
            fread(&val, sizeof(double), 1, in);
            records[i*num_samples + j] = (Scalar)val;
        }
        fseek(in, (num_all_samples-samples_end)*bytes_per_sample, SEEK_CUR);
    }
    fclose(in);
    return 0;
}

int km::WriteCluFile(const QString& filename, int k, const int* assignments, const Scalar* centers,
                     int num_records, int num_samples)
{
    QString msg;
    FILE* out = fopen(filename.toAscii(), "ab");
    if (!out)
    {
        msg = "File name " + filename + " not valid, no clustering done.";
        cerr << msg.toStdString() << endl;
        fclose(out);
        return -1;
    }

    int kbytes = fwrite(&k, sizeof(int), 1, out);
    int abytes = fwrite(assignments, sizeof(int), num_records, out);
    int cbytes = fwrite(centers, sizeof(double), k*num_samples, out);
    fclose(out);

    msg = QString("Wrote clustering %1 to disk, %2, %3, %4").arg(k).arg(kbytes).arg(abytes).arg(cbytes);
    cerr << msg.toStdString() << endl;

    return 0;
}
