#include <QtCore/QCoreApplication>

#include "../../clus_globals.h"
#include "globals.h"
#include "KMeans.h"
#include "threadmgr.h"

/* David Arthur's kmeans++ clustering algorithm.  The arguments this expects are
  1. Filename base
  2. Electrode number
  3. k (number of clusters)
  4. attempts (number of attempts)
  */

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);
//    qRegisterMetaType<Scalar>("Scalar");

    QStringList args = a.arguments();
    QString file_name_base, id, elc;
    int window_begin, window_end, samples_begin, samples_end;
    int k_min, k_max, num_attempts;
    if (args.length()<5) //For debugging
    {
        const char* cmd = "rm ~ch_090928lipaM.raw.10.clu.1;";
        system(cmd);

        file_name_base = "~ch_090928lipaM.raw.0";
        id = "10";
        elc = "1";
        window_begin = 0;
        window_end = 4595;
        samples_begin = 0;
        samples_end = 64;
        k_min = 2;
        k_max = 3;
        num_attempts = 20;
    }
    else
    {
        file_name_base = args.at(1);
        id = args.at(2);
        elc = args.at(3);
        window_begin = args.at(4).toInt();
        window_end = args.at(5).toInt();
        samples_begin = args.at(6).toInt();
        samples_end = args.at(7).toInt();
        k_min = args.at(8).toInt();
        k_max = args.at(9).toInt();
        num_attempts = args.at(10).toInt();
    }
        //Note that k==1 is accepted for completeness, even though you don't need
        //KMeans to put all waveforms into a single cluster

    cerr <<  "File name: " << file_name_base.toStdString() << endl;
    cerr <<  "ID: " << id.toStdString() << endl;
    cerr <<  "Electrode: " << elc.toStdString() << endl;
    cerr <<  "Window begin: " << window_begin << endl;
    cerr <<  "Window end: " << window_end << endl;
    cerr <<  "Sample begin: " << samples_begin << endl;
    cerr <<  "Sample end: " << samples_end << endl;
    cerr <<  "Number of clusters, min: " << k_min << endl;
    cerr <<  "Number of clusters, max: " << k_max << endl;
    cerr <<  "Number of attempts: " << num_attempts << endl;

    int num_records, num_samples;
    Scalar* records;

    QString inputfile = file_name_base + ".wfm." + elc;
    int fail = km::LoadPointsFromFile(inputfile,
                                      window_begin, window_end, samples_begin, samples_end,
                                      num_records, num_samples, records);
    if (fail)
    {
        cerr << "Could not load file, KMeans classification failed!" << endl;
        return fail;
    }

//    int idx = file_name_base.lastIndexOf(".");
//    file_name_base = file_name_base.left(idx);
    QString clu_file = file_name_base + ".wfm." + elc + "." + id + ".km";

    ThreadMgr tmgr(clu_file);
    fail = tmgr.RunKMeansPlusPlus(num_records, k_min, k_max, num_samples, records, num_attempts);

    if (fail)
    {
        cerr << "Could not classify data, KMeans classification failed!" << endl;
        return fail;
    }

    cerr << "Finished k-means clustering of " << inputfile.toStdString() << " successfully, wrote results to "
         << clu_file.toStdString() << endl;
//    return a.exec();
}
