#ifndef THREADMGR_H
#define THREADMGR_H

#include <QThread>
#include <QMutex>
#include <QMap>
#include <QVector>
#include <QPair>
#include "globals.h"
#include "kmeansto.h"

class Sleeper : public QThread
{
public:
    static void sleep_us(unsigned long us)
    {
        QThread::usleep(us);
    }
};
class ThreadMgr : public QObject
{
    Q_OBJECT

    public:
        typedef QPair<const int*, const Scalar*> ClusPair;

        ThreadMgr(const QString& clu_file);
        int RunKMeansPlusPlus(int num_records, int k_min, int k_max, int num_samples,
                          Scalar* records, int num_attempts);
    signals:
        void Start();

    public slots:
        void KMeansFinished(int k, const int* assignments, const Scalar* centers);

    private:
        void SetComplete() { _mutex.lock(); _incomplete = false; _mutex.unlock(); }
        bool IsIncomplete() { bool is_incomplete; _mutex.lock(); is_incomplete = _incomplete;
                              _mutex.unlock(); return is_incomplete; }
        int WriteCluster();

        QString _cluFile;
        volatile bool _incomplete;

        int _numRecords, _numSamples, _kMin, _kMax;
        int _numThreadsRemaining, _result;
        QMap<int, ClusPair> _clusters;

        QVector<QThread*> _threads;
        QMutex _mutex;
};

#endif // THREADMGR_H
