#include "channelbatch.h"
#include "../gui/mainwindow.h"

ChannelBatch::ChannelBatch(WavesFilePtr waves_file, MainWindow* mw)
    : QObject(), _cmed(ClusterMediator::Instance()), _mainWindow(mw), _thread(nullptr),
      _wavesFile(waves_file)

{
}
ChannelBatch::~ChannelBatch()
{
    if (_thread) _thread->detach();
}

void ChannelBatch::Run()
{
    _thread = new std::thread( std::bind(&ChannelBatch::Process, this) );
}

void ChannelBatch::Process()
{
    try
    {
        const std::vector<short>& electrodes = _wavesFile->VElectrodes();
        short electrode0 = electrodes.at(0);
        for (auto& it : electrodes)
        {
            if (it==electrode0) continue;
                //This is done in MainWindow::OnActionLoadWaveformFileTriggered
            _wavesFile->GenerateTransforms(it);
            emit PreprocFinished(_wavesFile, it);
            QApplication::processEvents();
        }
    }
    catch (std::exception& e)
    {
        Log(e.what());
    }
}

