#ifndef KMEANSTO_H
#define KMEANSTO_H

#include "clusmethodtobase.h"
#include "../../memrep/parameters/cluster/kmeanspars.h"
#include "../../file/input/kmeansfilein.h"
#include "../../file/output/mclufileout.h"

class KMeansTO : public ClusMethodTOBase
{
    public:
        KMeansTO(wv::CLUSTER_ID cid, wv::TRANSFORM transform,
                 MatCPtr waves, const std::string& wfm_file);
        virtual ~KMeansTO() override {}
        virtual void Start() override;
        virtual std::string Name() const override { return "KMeans"; }
        virtual wv::CLUSTER_METHOD Method() const override { return wv::KMEANS; }

    protected slots:
        virtual void LoadClassification(int exit_code) override;

    private:
        //Following http://en.wikipedia.org/wiki/Determining_the_number_of_clusters_in_a_data_set
        wv::float_t Distortion(const wv_vector_int* assignment, const wv_matrix* centers, int k);
        int MaxJump(const std::map<int, KMPair>* clusters);

        int _numClusterings, _numSamplesUsed;

        /* Following "Estimating the number of clusters in a data set via the gap statistic", Tibshirani
          R, Walther G., Hastie H, J. R. Statistical Society B (2001) */
        //The gap statistic looks looks very expensive...
        //Can get it from R, http://rgm2.lab.nig.ac.jp/RGM2/func.php?rd_id=clusterSim:index.GAP ?
        wv::float_t Dispersion(const int* assignment, int c);
        wv::float_t W(const int* assignment, int k);
};

#endif // KMEANSTO_H
