#ifndef DUNN_H
#define DUNN_H

#include "../globals/globals.h"
#include "../globals/log.h"
#ifdef USE_GSL
#include "../analysis/gsl_utils/mgsl.h"
#endif

//We encapsulate this in a class in case we ever want the user to be able to choose between
//different statistical analyses, then this will implement the strategy pattern.  Note this is
//a *variation* on the Dunn analysis, which doesn't return a value corresponding to an entire
//clustering but rather a value for each cluster.

//For a single cluster, this function returns the amplitude divided by the maximum variance
//among the samples.
class Dunn
{
    public:
        Dunn(MatCPtr waves, const wv_vector_int* clustering, const std::vector<int>& clusters,
             int j_begin, int j_end);
        void Calculate();

        const std::vector<wv::float_t>& Indexes() const { return _dunnIndexes; }

    private:
        void Calculate1(); //For when there is a single cluster

        std::vector<int> _clusters;
        std::vector<wv::float_t> _dunnIndexes;
        int _selectedSamplesBegin, _selectedSamplesEnd;
        const wv_vector_int* _vClusters;
        MatCPtr _waves;
};

#endif // DUNN_H
