#include "covarianceto.h"
#include "covthreadmgr.h"

CovarianceTO::CovarianceTO(CovThreadMgr* thread_mgr, size_t num_rows)
    : _threadMgr(thread_mgr), _isRunning(false), _numRows(num_rows)
{
}

void CovarianceTO::Start()
{
    _isRunning = true;
    while (_isRunning)
    {
        _mutex.lock();
        if (_queue.isEmpty())
        {
            _mutex.unlock();
            continue;
        }
        CovSpec cov_spec = _queue.dequeue();
        double covariance = gsl_stats_covariance(cov_spec.data1,
                                                 cov_spec.stride1,
                                                 cov_spec.data2,
                                                 cov_spec.stride2,
                                                 _numRows);
        _mutex.unlock();
        _threadMgr->SetResult(cov_spec.j1, cov_spec.j2, covariance);
    }
    emit ThreadComplete();
}

void CovarianceTO::Enqueue(const CovSpec& cov_spec)
{
    _mutex.lock(); //No QMutexLocker, for speed
    _queue.enqueue(cov_spec);
    _mutex.unlock();
}

void CovarianceTO::Finished()
{
    //NB We're mutex protected because the original call here came from CovarianceTO::Start()
    // -> CovThreadMgr::SetResult
    _isRunning = false;
    if (!_queue.isEmpty())
    {
        _queue.clear();
        qDebug() << "INTERNAL ERROR: Covariance thread finished before queue empty, please contact" \
            " software maintainer.";
    }
}
