#ifndef CLUSTERMGR_H
#define CLUSTERMGR_H

#include "threadmgr.h"

//For the clusterings--just one clustering when launched from WaveSpaceGui, multiple when doing batch
//processing.  Using the ThreadMgr framework is still essential however since multiple clusterings can
//be occuring at once.
class ClusterMgr : public ThreadMgr
{
    Q_OBJECT
    public:
        explicit ClusterMgr(const std::vector<wv::CLUSTER_ID>& cids);

        virtual WorkerTO* CreateWorkerTO(int thd_num) override;
        virtual void OnThreadFinished(void *) override;

        bool HasCID(wv::CLUSTER_ID cid) const;
        int NumThreadsRemaining() const { return _numThreadsRemaining; }

    private:
#ifndef Q_OS_WIN
        ClusterMgr(const ClusterMgr&) = delete;
        ClusterMgr operator=(const ClusterMgr&) = delete;
#else
        ClusterMgr(const ClusterMgr&);
        ClusterMgr operator=(const ClusterMgr&);
#endif

        std::vector<wv::CLUSTER_ID> _cids;

        int _numThreadsRemaining;
};

typedef std::shared_ptr<ClusterMgr> ClusterMgrPtr;

#endif // CLUSTERMGR_H
