#include "histscattermgr.h"
#include "../worker/histscatterto.h"

HistScatterMgr::HistScatterMgr(HistWidget* hist_widget, HistSpec hist_spec)
    : ThreadMgr(qMax( QThread::idealThreadCount()-1, 1), 2000),
      _histWidget(hist_widget), _histSpec(hist_spec), _matAccept(hist_widget->_matAccept), _matUnaccept(hist_widget->_matUnaccept)
{
    _numRecords = _histSpec.xdata_vec->size;
    _recordsPerThread = _numRecords / NumThreads();
}
WorkerTO* HistScatterMgr::CreateWorkerTO(int thd_num)
{
    const wv_vector* xdata_vec = _histSpec.xdata_vec,
            * ydata_vec = _histSpec.ydata_vec;
    wv_vector* xvec = wv_vector_alloc(xdata_vec->size),
            * yvec = wv_vector_alloc(ydata_vec->size);
    wv_vector_memcpy(xvec, xdata_vec);
    wv_vector_memcpy(yvec, ydata_vec);
        //xvec, yvec deleted by HistScatterTO

    int row_start = thd_num * _recordsPerThread,
            row_end = (thd_num == NumThreads()-1)
            ? _numRecords
            : row_start + _recordsPerThread;
    _histSpec.start_row = row_start;
    _histSpec.end_row = row_end;
    HistScatterTO* to = new HistScatterTO(thd_num, _histWidget, _histSpec, xvec, yvec);
    return to;
}

void HistScatterMgr::OnThreadFinished(void* result)
{
    auto results_pair = static_cast< std::pair<wv_matrix_int*, wv_matrix_int*>* >(result);
    wv_matrix_int* mat_accept = results_pair->first,
            * mat_unaccept = results_pair->second;

    wv_matrix_int_add(_matAccept, mat_accept);
    wv_matrix_int_add(_matUnaccept, mat_unaccept);

    wv_matrix_int_free(mat_accept);
    wv_matrix_int_free(mat_unaccept);
    delete results_pair;
}
