#ifndef SPLINEMGR_H
#define SPLINEMGR_H

#include "threadmgr.h"

class SplineMgr : public ThreadMgr
{
    public:
        struct SplineSpec
        {
            SplineSpec(MatCPtr waves_) : waves(waves_), spline(NULL), height(NULL), width(NULL), slope(NULL)
            {}
            MatCPtr waves;
            int i_start, i_end;

            int left_buf, right_buf, spline_order, num_breakpts;
            wv::float_t grain;
            bool is_raw;

            //These vectors hold interpolated values for fine-grained calculation of height, width
            wv_vector* lmax_vec, * rmax_vec, * min_vec;

            //For b-splines
            gsl_bspline_workspace* bspline_ws;
            wv_vector* breakpts,
                    * B;
            wv_matrix* X;

            //For workspace
            gsl_multifit_linear_workspace* multifit_ws;
            wv_matrix* cov;
            wv_vector* coeffs;

            //Output data structures--these are actually read from the struct in SplineTO
            wv_matrix* spline, * height, * width, * slope;
        };

        explicit SplineMgr(SplineSpec& spec); //Non const because it also contains writeable members.  Could
            //break this in two, but don't want to bother...

        virtual WorkerTO* CreateWorkerTO(int thd_num) override;
        wv_matrix* Spline() const { return _spec.spline; }
        wv_matrix* Height() const { return _spec.height; }
        wv_matrix* Width() const { return _spec.width; }
        wv_matrix* Slope() const { return _spec.slope; }

    private:
        SplineSpec& _spec;
        int _recordsPerThread;
};

#endif // SPLINEMGR_H
