#include "diffto.h"

DiffTO::DiffTO(int thd_num, DiffMgr::DiffSpec& diff_spec)
    : WorkerTO(thd_num), _diffSpec(diff_spec)
{
}

void DiffTO::Start()
{
    //Get info from spec
    const wv_matrix* raw = _diffSpec.raw;
    wv_matrix* raw_d = _diffSpec.raw_d,
            * slope_d = _diffSpec.slope_d;
    bool do_smooth = _diffSpec.do_smooth;
    wv::float_t us_per_samp = _diffSpec.us_per_samp;
    int num_samples = raw->size2,
            num_samples_d = num_samples - 1,
            num_samples_d1 = num_samples_d - 1;

    wv_vector* row = wv_vector_alloc(num_samples);
    wv::float_t* d;
    size_t i_end = _diffSpec.i_end;
    for (size_t i=_diffSpec.i_start; i<i_end; ++i)
    {
        if (do_smooth)
        {
            wv_matrix_get_row(row, raw, i);
            d = row->data;
        }
        else
            d = wv_matrix_const_row(raw, i).vector.data;

        if (do_smooth)
        {
            d[0] = (d[0] + d[1])/2.0;
            for (int j=1; j<num_samples_d1; ++j)
                d[j] = (d[j-1] + d[j] + d[j+1])/3.0;
            d[num_samples_d1] = (d[num_samples_d1-1] + d[num_samples_d1])/2.0;
        }
            // ***Note that this is feeding the smoothed values back into the algorithm, not really
            //the right way to do it, but seems to be working well enough

        //We know raw_d is row major and size2==tda
        wv_vector_view drow = wv_matrix_row(raw_d, i);
        memcpy(drow.vector.data, d+1, num_samples_d*sizeof(wv::float_t));
        wv_blas_axpy(num_samples_d, -1.0, d, 1, drow.vector.data, drow.vector.stride);

        //Get the slope
        wv::float_t min, max;
        size_t imin, imax;
        wv_vector_const_view dcrow = wv_matrix_const_row(raw_d, i);
        wv_vector_minmax_index(&dcrow.vector, &imin, &imax);
        min = wv_vector_get(&dcrow.vector, imin);
        max = wv_vector_get(&dcrow.vector, imax);
        wv::float_t slope = (imin==imax) ? 0 : (max-min)
                                      / (abs((int)(imax-imin))*us_per_samp);
        wv_matrix_set(slope_d, i, 0, slope);
    }
    wv_vector_free(row);

    WorkFinished();
}
