#include "fourierworkerto.h"

FourierWorkerTO::FourierWorkerTO(int thd_num, FourierMgr::FourierSpec& fourier_spec)
    : WorkerTO(thd_num), _fourierSpec(fourier_spec)
{
}

void FourierWorkerTO::Start()
{
    wv_matrix* transform = _fourierSpec.transform;
    int num_samples = transform->size2;

    //Initialize fft objects
    gsl_fft_real_wavetable* wavetable = gsl_fft_real_wavetable_alloc(num_samples);
    gsl_fft_real_workspace* workspace = gsl_fft_real_workspace_alloc(num_samples);

    int i_end = _fourierSpec.i_end;
    for (int i=_fourierSpec.i_start; i<i_end; ++i)
    {
        //Do the fft
        wv_vector_view row = wv_matrix_row(transform, i);
        gsl_fft_real_transform(row.vector.data, row.vector.stride, num_samples,
                                     wavetable, workspace);
        // ***Not clear that GSL's radix2 algorithms are faster than mixed-radix, see pg. 187 of the
        //GSL Reference manual 3rd ed.

        //### Re-pack the data, phase followed by magnitude is probably best
    }

    gsl_fft_real_wavetable_free(wavetable);
    gsl_fft_real_workspace_free(workspace);
    WorkFinished();
}
