#include "haarworkerto.h"

#ifndef USE_GSL
std::mutex HaarWorkerTO::_mutex;
#endif

HaarWorkerTO::HaarWorkerTO(int thd_num, HaarMgr::HaarSpec haar_spec)
    : WorkerTO(thd_num), _haarSpec(haar_spec)
{
}

void HaarWorkerTO::Start()
{
    wv_matrix* transform = _haarSpec.transform;
    int num_samples = transform->size2;

    //Initialize wavelet objects
#ifdef USE_GSL
    bool is_centered = _haarSpec.is_centered;
    int size = _haarSpec.size;
    gsl_wavelet* wavelet;
    if (size==-1 && is_centered)
        wavelet = gsl_wavelet_alloc(gsl_wavelet_haar_centered, 2);
    else if (size==-1)
        wavelet = gsl_wavelet_alloc(gsl_wavelet_haar, 2);
    else if (size < 100 && is_centered)
        wavelet = gsl_wavelet_alloc(gsl_wavelet_daubechies_centered, size);
    else if (size < 100)
        wavelet = gsl_wavelet_alloc(gsl_wavelet_daubechies, size);
    else if (is_centered)
        wavelet = gsl_wavelet_alloc(gsl_wavelet_bspline_centered, size);
    else
        wavelet = gsl_wavelet_alloc(gsl_wavelet_bspline, size);
    gsl_wavelet_workspace* workspace = gsl_wavelet_workspace_alloc(num_samples);

    //Compute the wavelets
    size_t i_end = _haarSpec.i_end;
    for (size_t i=(size_t)_haarSpec.i_start; i<i_end; ++i)
    {
        wv_vector_view v = wv_matrix_row(transform, i);
        gsl_wavelet_transform_forward(wavelet, v.vector.data, v.vector.stride, num_samples, workspace);
    }

    gsl_wavelet_free(wavelet);
    gsl_wavelet_workspace_free(workspace);
#else
    //Have to lock it up, because lwtl makes use of global variables.
    _mutex.lock();
    msl::lwtl_wavelet_init(num_samples);
    wv::float_t* d = transform->data;

    size_t i_end = _haarSpec.i_end;
    for (size_t i=(size_t)_haarSpec.i_start; i<i_end; ++i)
        msl::lwtl_wavelet_forward( d + i*num_samples ); // *** Row major is critical here

    msl::lwtl_wavelet_close();
    _mutex.unlock();
#endif
    WorkFinished();
}
