#include "histwgtto.h"

HistWgtTO::HistWgtTO(int thd_num, uchar* pixels, const HistWidget* hist_widget, int jstart, int jend, wv::float_t max_val) : WorkerTO(thd_num),
      _pixels(pixels), _histWidget(hist_widget), _scaleSpec(hist_widget->_scaleSpec),
    _histAccept(hist_widget->_histAccept), _histUnaccept(hist_widget->_histUnaccept),
      _histWidth(hist_widget->width()), _jStart(jstart), _jEnd(jend), _maxVal(max_val)
{
}

void HistWgtTO::Start() //slot
{
    int j_range = _jEnd - _jStart;
    wv_vector* xvec = wv_vector_alloc(j_range),
            * yvec = wv_vector_alloc(j_range);
    for (int i=0, j=_jStart; j<_jEnd; ++j, ++i)
        wv_vector_set(yvec, i, j);

    //Get min, max for calculating exclusion
    ClusterMediator& cmed = ClusterMediator::Instance();
    WSGSpec& wsg_spec = cmed.GetCluSpecPtr(wv::DEFAULT_CID)->GetWSGSpec();
    wv::float_t xmin = wsg_spec.Min(WSGSpec::X), xmax = wsg_spec.Max(WSGSpec::X),
            ymin = wsg_spec.Min(WSGSpec::Y), ymax = wsg_spec.Max(WSGSpec::Y);

    _scaleSpec->PixToValYVec(yvec);

    for (int i=0; i<_histWidth; ++i)
    {
        wv_vector_set_all(xvec, i);
        _scaleSpec->PixToValXVec(xvec);

        for (int j=_jStart, k=0; j<_jEnd; ++j, ++k)
        {
            wv::float_t xzoom = wv_vector_get(xvec, k),
                    yzoom = wv_vector_get(yvec, k);

            if ( xzoom>=xmin && xzoom<xmax && yzoom>=ymin && yzoom<ymax )
            {
                size_t ibin, jbin;
                wv_histogram2d_find(_histAccept, xzoom, yzoom, &ibin, &jbin);
                int green = (255.0 * wv_histogram2d_get(_histAccept, ibin, jbin)
                                      / _maxVal + 0.5);
                int white = qMin(255.0, 255.0 * wv_histogram2d_get(_histUnaccept, ibin, jbin)
                                      / _maxVal + 0.5);

                int idx = (j*_histWidth + i) * 4;
                _pixels[idx+3] = 0xff;
                _pixels[idx+2] = (uchar)white;
                _pixels[idx+1] = (uchar)qMin(255, white+green);
                _pixels[idx] = (uchar)white;
                    //Don't understand why but reverse order is required
            }
        }
    }

    wv_vector_free(xvec);
    wv_vector_free(yvec);

    WorkFinished();
}
