#include "splineto.h"

SplineTO::SplineTO(MatCPtr waves, wv::float_t us_per_samp, const std::string& file_name, short electrode, ParamsBase* pars, bool is_raw)
    : TransformTOBase(waves, file_name, electrode, pars, is_raw),
      _usPerSamp(us_per_samp), _splineOrder(4), _numBreakpts(11), _leftBuf(3), _rightBuf(5), _grain(256)
{
//    MakeProgressBar(file_name);
}

void SplineTO::Start()
{
    //Now put everything in the spec struct, to send to the threads via the SplineMgr
    SplineMgr::SplineSpec spec( Waves() );
    spec.left_buf = _leftBuf;
    spec.right_buf = _rightBuf;
    spec.grain = _grain;
    spec.is_raw = IsRaw();
    spec.spline_order = _splineOrder;
    spec.num_breakpts = IsRaw()
            ? boost::lexical_cast<int>( session_profile.Par("Session", "Num Spline Breakpoints") )
            : boost::lexical_cast<int>( session_profile.Par("Session", "Num Spline Breakpoints") )+1;

    SplineMgr mgr(spec);
    mgr.StartThreads();
    mgr.Wait();

    //Scale to replace sample # with actual duration
    wv_matrix_scale(mgr.Width(), _usPerSamp);
    wv_matrix_scale(mgr.Slope(), 1.0/_usPerSamp);

    //Set data members to the transforms struct
    TransformGroup* transforms = new TransformGroup;
    transforms->spline = MatPtr( mgr.Spline(), wv_matrix_free );
    transforms->height = MatPtr( mgr.Height(), wv_matrix_free );
    transforms->width = MatPtr( mgr.Width(), wv_matrix_free );
    transforms->slope = MatPtr( mgr.Slope(), wv_matrix_free );

    AnalysisFinished(transforms);
}

void SplineTO::AnalysisFinished(void* transform)
{
    TransformGroup* transform_gp = static_cast<TransformGroup*>(transform);
//    WriteTempFile( transform_gp->height, wv::HEIGHT );
//    WriteTempFile( transform_gp->width, wv::WIDTH );
//    WriteTempFile( transform_gp->slope, IsRaw() ? wv::SLOPE : wv::SLOPE_D );
//    WriteTempFile( transform_gp->spline, IsRaw() ? wv::SPLINE : wv::SPLINE_D );
    auto pair = new std::pair<wv::TRANSFORM, MatPtr>( Transform(), transform_gp->spline );
    TransformTOBase::AnalysisFinished(pair);
}
void SplineTO::AnalysisFinished(const void* transform) const
{
    const TransformGroup* transform_gp = static_cast<const TransformGroup*>(transform);
//    WriteTempFile( transform_gp->height, wv::HEIGHT );
//    WriteTempFile( transform_gp->width, wv::WIDTH );
//    WriteTempFile( transform_gp->slope, IsRaw() ? wv::SLOPE : wv::SLOPE_D );
//    WriteTempFile( transform_gp->spline, IsRaw() ? wv::SPLINE : wv::SPLINE_D );
    auto pair = new std::pair<wv::TRANSFORM, wv_matrix*>( Transform(), transform_gp->spline.get() );
    TransformTOBase::AnalysisFinished(pair);
}

