#ifndef SPLINETO_H
#define SPLINETO_H

#include "transformtobase.h"
#include "../../analysis/threads/mgr/splinemgr.h"

//NB for expedience, spline, spline derivative, height, width, and slope are all calculated here.
class SplineTO : public TransformTOBase
{
    Q_OBJECT

    public:
        struct TransformGroup
        {
            MatPtr spline, height, width, slope;
        };

        SplineTO(MatCPtr waves, wv::float_t us_per_samp, const std::string& file_name, short electrode, ParamsBase* pars, bool is_raw);
        virtual ~SplineTO() override {}

        virtual void AnalysisFinished(void *analysis) override;
        virtual void AnalysisFinished(const void *analysis) const override;

        virtual std::string Name() const override { return "ERROR: Multiple transforms, can't use this function"; }
        virtual wv::TRANSFORM Transform() const override { return (IsRaw()) ? wv::SPLINE : wv::SPLINE_D; }

    public slots:
        virtual void Start() override;

    private:
        wv::float_t _usPerSamp; //Need to calculate width, slope in actual us, not arbitrary sample numbers
        const int _splineOrder, _numBreakpts,
                _leftBuf, _rightBuf,
            //The buffers are to ensure that there are enough samples between imin and the boundaries to insert
            //the other points.
                _grain; //How finely to interpolate around the min, max

};

#endif // SPLINETO_H
