#include "mclufilein.h"

MCluFileIn::MCluFileIn(const std::string& file_name, const boost::dynamic_bitset<>& window)
    : InputFileBase(file_name, true), _window(window)
{
}
MCluFileIn::~MCluFileIn()
{
    delete static_cast<Classification*>( Data() );
}

int MCluFileIn::Load()
{
    std::vector<Clustering> vclusters;
    int num_records = -1,
            best;

    //Now get the cluster values, both putting them in the big num_records-length
    FILE* clu = File();
    if (clu)
    {
        int num_clusterings;
        fread(&num_records, sizeof(int), 1, clu);
        fread(&num_clusterings, sizeof(int), 1, clu);
        fread(&best, sizeof(int), 1, clu);

        for (int i=0; i<num_clusterings; ++i)
        {
             wv::float_t level;
             fread(&level, sizeof(wv::float_t), 1, clu);
             wv_vector_int* clustering = wv_vector_int_alloc(num_records);
             wv_vector_int_fread(clu, clustering);

             vclusters.push_back( Clustering(level, clustering) );
        }
    }
    else
    {
        Log("WARNING: Invalid file name, " + Path() + ", no records loaded.");
        return -1;
    }

    //Now create the classification
    SetData( new Classification(best, vclusters) );

    return num_records;
}
