#include "profilein.h"

ProfileIn::ProfileIn() : InputFileBase(FileBase::UserPath() + "/profile.txt", false)
{
}
ProfileIn::~ProfileIn()
{
    Profile* data = static_cast<Profile*>( Data() );
    delete data;
}

int ProfileIn::Load()
{
    FILE* fp = File();
    if (!fp) throw BadFileName("ProfileIn::Load 1", Path());
    QFile qfp;
    qfp.open(fp, QIODevice::ReadOnly);
    if (!qfp.isOpen()) throw BadFileName("ProfileIn::Load 2", Path());

    Profile* pdata = new Profile();

    //Initialize default parameters here, because not all parameters in sessionpars et al.
    //will necessarily be present in the user's profile
    SessionPars sp;
    AutoClassPars acp;
    KMeansPars kmp;
    KKParameters kkp;
    SPCParameters spcp;
    pdata->AddParList( sp.Name(), sp.ParList() );
    pdata->AddParList( acp.Name(), acp.ParList() );
    pdata->AddParList( kmp.Name(), kmp.ParList() );
    pdata->AddParList( kkp.Name(), kkp.ParList() );
    pdata->AddParList( spcp.Name(), spcp.ParList() );

    std::string current_list("");
    while (!qfp.atEnd())
    {
        std::string line = qfp.readLine().data();
        boost::algorithm::trim(line);
        if (line.empty() || line.at(0) == '#') continue;
        if (line.at(0) == '@')
        {
            if (line=="@@User Transforms@@")
            {
                WaveData::ReadUTSignatures(fp);
                break;
            }
            std::string list_name = line.substr(1);
            current_list = list_name;
        }
        else
        {
            if (current_list.empty())
            {
                Log("ERROR: parameter file " + Path() + " is corrupted, profile not loaded");
                return 0;
            }

            StringVec param;
            size_t pos = line.find_first_of(':');
            std::string key = line.substr(0, pos),
                    value = line.substr(pos+1);
            boost::algorithm::trim(key); boost::algorithm::trim(value);
            pdata->AddPar(current_list, Param(key, value) );
        }
    }

    session_profile = *pdata;

    std::string data_dir = session_profile.Par("Session", "Data Directory"),
            ws_data_dir = session_profile.Par("Session", "WS Data Directory");
    FileBase::SetUserDataDir(data_dir);
    FileBase::SetUserWSDataDir(ws_data_dir);

    SetData(pdata);
    return -1;
}
