#ifndef NEVFILE_H
#define NEVFILE_H

#include "waveformsinbase.h"

//This spec is taken from Neuroshare Native File Format Specification Rev 0.9d
//at http://neuroshare.sourceforge.net/download.shtml#NSN-Documentation

//COULD NOT FIND a simple document with the file structure of a NEV file, leaving this alone for now

class NevFile : public WaveformsInBase
{
    typedef unsigned int uint;
    typedef unsigned short ushort;
    typedef unsigned char uchar;

    static const int NS_PID_COMMENT;
    static const int NS_PID_CONFLOG;

    typedef struct
    {
        char        FileTypeId[8];
        uchar       FileSpec[2];
        ushort      AdditionalFlags;
        uint        BytesInHeaders, //This is the entirety of bytes used by the file
                        //in sections 1 and 2.
                    BytesInDataPackets,
                    TimeResolutionOfTimeStamps,
                    TimeResolutionOfSamples;
        ushort      TimeOrigin[8];
        char        ApplicationToCreateFile[32];
        char        CommentField[256];
        uint        NumberofExtendedHeaders;
    } ns_NEURALEV;

    typedef struct
    {
        char        PacketId[8];
        char        InformationField[24];
    } ns_EXTENDED_HEADER;

    struct DataPacket
    {
        uint        TimeStamp;
        ushort       PacketId;
        double*     data; //Looks like you don't know how long this is until runtime
    };

    public:
        NevFile(const std::string& full_path);
        virtual ~NevFile() override {}

        virtual int Load() override;

    private:
        std::vector<ns_EXTENDED_HEADER> _extHeaders;
};

#endif // NEVFILE_H
