#include "acparamsout.h"

ACParamsOut::ACParamsOut(const std::string& file_name, PARAM_TYPE param_type, ParamsCPtr autoclass_pars,
                         int num_all_samples)
    : OutputFileBase(file_name, false, NULL, boost::dynamic_bitset<>()), _paramType(param_type),
      _autoclassPars(autoclass_pars), _numAllSamples(num_all_samples)
{
}

void ACParamsOut::Save() const
{
    FILE* fp = File();

    int num_pars = _autoclassPars->NumPars();
    bool found_sparams = false,
            found_rparams = false;
    for (int i=0; i<num_pars; ++i)
    {
        std::string par_key = _autoclassPars->Key(i);
        if (par_key == "WS-PARAMS END")
            found_sparams = true;
        if (!found_sparams || par_key == "WS-PARAMS END") continue;

        if (found_rparams || par_key == "S-PARAMS END")
        {
            if (_paramType==SEARCH) break;
            found_rparams = true;
        }
        if (par_key == "S-PARAMS END") continue;
        if ( (found_rparams && _paramType==SEARCH) || (!found_rparams && _paramType==RESULTS) )
            continue;

        if ( ut::Contains(par_key, "-s") || ut::Contains(par_key, "-r")) par_key.erase( par_key.size()-2 );
            //These two lines are necessary because some r-params and s-params have the same name.
            //So in the parameter file they are given the corresponding suffixes to distinguish them.

        char key[256], value[256];
        std::string par_value = _autoclassPars->Value(i);
        if (par_key == "max_duration")
        {
            wv::float_t usec_per_sample = boost::lexical_cast<wv::float_t>(par_value);
            int max_duration = (int)(_numAllSamples * usec_per_sample / 1000000.0 + 1.0); //round up
            par_value = std::to_string(max_duration);
        }
        strcpy(key, par_key.c_str());
        strcpy(value, par_value.c_str());
        fprintf(fp, "%s = %s\n", key, value);
    }
        //So basically the idea here is that s-params and r-params go into different files,
        //and so are represented in different blocks in the parameter file
}
