#include "profileout.h"

ProfileOut::ProfileOut(const std::string& list_name, const ParamVec& param_list)
    : OutputFileBase(FileBase::UserPath() + "/profile.txt", false, NULL, boost::dynamic_bitset<>()),
      _listName(list_name), _paramList(param_list)
{
}

void ProfileOut::Save() const
{
    FILE* fp = File();
    if (fp)
    {
        const StringVec& param_lists = session_profile.ParLists();
        bool found_list = false; //The idea here is to allow new lists (resulting from new cluster methods, etc.)
            //to be added to the profile easily.
        auto param_lists_end = param_lists.end();
        for (auto it=param_lists.begin(); it!=param_lists_end; ++it)
            if (*it == _listName)
            {
                char session[256];
                strcpy(session, std::string("@" + _listName + "\n").c_str());
                fprintf(fp, session);
                auto param_list_end = _paramList.end();
                for (auto itp=_paramList.begin(); itp!=param_list_end; ++itp)
                {
                    char par[256];
                    strcpy(par, std::string("\t" + itp->first + ":\t" + itp->second + "\n").c_str());
                    fprintf(fp, par);
                }
                found_list = true;
            }
            else
                session_profile.Write(fp, *it);

        if (!found_list && !_paramList.empty()) Write(fp);

        fprintf(fp, "@@User Transforms@@\n");
        WaveData::WriteUTSignatures(fp);
    }
    else
        Log("WARNING: Invalid file name, " + Name() + ", no records loaded.");
}

void ProfileOut::Write(FILE* fp) const
{
    char session[256];
    strcpy(session, std::string("@" + _listName + "\n").c_str());
    fprintf(fp, session);
    for (auto itp : _paramList)
    {
        char par[256];
        strcpy(par, std::string("\t" + itp.first + ":\t" + itp.second + "\n").c_str());
        fprintf(fp, par);
    }
}
