#ifndef REXEFILE_H
#define REXEFILE_H

#include "outputfilebase.h"

#define REX_ERRLOG
#undef REX_ERRLOG

class RexEFile : public OutputFileBase
{
    public:
        RexEFile(const std::string& old_efile, const std::string& name, const WaveData* data, const boost::dynamic_bitset<>& accepted);
            //Since reading the efile changes the state of the FILE pointer, efile can't (and
            //shouldn't) be const.
        virtual void Save() const override;

    private:
        static const int EHEADER_SZ;
        static const int REX_PANIC;
        struct RexEvent
        {
            RexEvent(ushort num, short cd, int t) : seq_num(num), code(cd), tim(t) {}
            RexEvent() : seq_num(0), code(-1), tim(-1) {}
            ushort seq_num;
            short code;
            uint tim;
        };
        RexEvent ReadRexEvent() const;

        std::string _EFileName; //Can be useful for debugging, with stat
        FILE* _EFile; //The *original* REX E file, *not* the one to which the data are
            //being written.  Remember that the file which is loaded is the M-file, so this
            //needs to be opened here.
#ifdef REX_ERRLOG
        mutable FILE* oldlogf;  //For debugging
#endif
};

#endif // REXEFILE_H
