#include "userclufileout.h"
#include "../../memrep/clustermediator.h"

UserCluFileOut::UserCluFileOut(const std::string& file_name, wv::CLUSTER_ID cid)
    : OutputFileBase(file_name, false, NULL, boost::dynamic_bitset<>()),
      _cid(cid)
{
    ClusterMediator& cmed = ClusterMediator::Instance();
    _class = cmed.GetClassification(_cid);
    _level = cmed.ClusLevel(_cid);

    Clustering clustering = _class->VClusters().at(_level);
    const wv_vector_int* vcluster = clustering.second;

    const boost::dynamic_bitset<>& is_scrubbed = cmed.GetWaveData(_cid)->Scrubbed();
        //This has the size of the *original* number of records.
    int num_all_records = is_scrubbed.size();
    _allClusters = wv_vector_int_alloc(num_all_records);
    for (int i=0, ct=0; i<num_all_records; ++i)
    {
        if (is_scrubbed.test(i))
            wv_vector_int_set(_allClusters, i, wv::NULL_CLUSTER);
        else
        {
            int clus_no = wv_vector_int_get(vcluster, ct++);
            wv_vector_int_set(_allClusters, i, clus_no);
        }
    }
}
UserCluFileOut::~UserCluFileOut()
{
    wv_vector_int_free(_allClusters);
}

void UserCluFileOut::Save() const
{
    FILE* clu = File();
    if (!clu) throw BadFileName("UserCluFileOut::Save", Name());

    const std::vector<int>& clusters = _class->Clusters(_level);
    fprintf(clu, "%d\n", (int)_allClusters->size);
    int num_clusters = clusters.size();
    for (int i=0; i<num_clusters; ++i)
        fprintf(clu, "%d\t", clusters.at(i));
    fprintf(clu, "\n");
    int code = WriteRows();
    if (code) Log("ERROR: Cluster analysis not written correctly to temporary file.");
}

int UserCluFileOut::WriteRows() const
{
    FILE* clu = File();
    int code = wv_vector_int_fprintf(clu, _allClusters, "%d");
    return code;
}
