#ifndef UTILS_H
#define UTILS_H

#include "boost_headers.h"
#include "qt_headers.h"
#include "stl_headers.h"

//Convenience functions, mainly to compensate for what is lost when switching from Qt to C++
namespace ut
{
    //boost
    void FillBitset(boost::dynamic_bitset<>& bitset, bool val, int begin, int end);
        //Fills [begin, end) with val.  Can't believe boost doesn't have this!

    //string functions--typically to replace some useful method from QString
    bool Contains(const std::string& str, char c);
    bool Contains(const std::string& str, const std::string& substr);
    bool EndsWith(const std::string& str, char c);
    bool EndsWith(const std::string& str, const std::string& substr);
    template<typename T>
    T Split(const std::string& str, const std::string& sep = " ");
    //###Actually a lot of these are in boost::algorithm, transition to that
    //replace with boost::algorithm::split

    //container functions
    template<typename T>
    int IndexOf(const std::vector<T>& vec, T val);
    template<typename T, typename U>
    std::vector<T> Keys(const std::map<T, U>& mp);
    template<typename T>
    T TakeFirst(std::list<T>& ls);
    template<typename T>
    std::list<T> ToList(const std::set<T>& st);
    template<typename T>
    std::vector<T> ToVector(const std::set<T>& st);

    //Qt conversion functions, usually **temporary** until Qt is entirely wiped out
    template<typename T>
    std::list<T> ToList(const QList<T>& qlist);
    QStringList ToQStringList(const StringList& slist);
    template<typename T>
    QVector<T> ToQVector(const std::vector<T>& vec);
    StringList ToStringList(const QStringList& qslist);
    template<typename T>
    std::vector<T> ToVector(const QVector<T>& vec);
}

StringList& operator<<(StringList& string_list, const std::string& str);
StringVec& operator<<(StringVec& string_vector, const std::string& str);

#endif // UTILS_H
