#include "wv.h"

const wv::CLUSTER_ID wv::DEFAULT_CID = 0;
const wv::CLUSTER_ID wv::NO_CLUSTER_ID = -1;

const wv::TRANSFORM_ID wv::DEFAULT_TID = 0;
const wv::TRANSFORM_ID wv::NO_TRANSFORM_ID = -1;

const int wv::DEFAULT_CLUSTER = 1;
const int wv::ZERO_CLUSTER = 0;
const int wv::NULL_CLUSTER = -1;
const int wv::REJECT_CLUSTER = -2;
const int wv::ACCEPT_CLUSTER = -3; //Needs to be the lowest of all possible cluster
    //numbers

#ifdef HAVE_EXTRA
const int wv::VERSION_MAJOR = 2;
const int wv::VERSION_MINOR = 0;
const int wv::VERSION_BUILD = 0;
#else
const int wv::VERSION_MAJOR = 1;
const int wv::VERSION_MINOR = 0;
const int wv::VERSION_BUILD = 5;
#endif
#ifdef HAS_CUDA
const std::string wv::VERSION_SUFFIX() { return "-cuda-dailybuild"; }
#else
#ifdef USE_GSL
const std::string wv::VERSION_SUFFIX() { return ""; }
#else
const std::string wv::VERSION_SUFFIX() { return "-dailybuild"; }
#endif
#endif
std::string wv::TransformName(TRANSFORM transform)
{
    std::string tr;
    switch (transform)
    {
        case wv::USER:
            tr = "user";
            break;
        case wv::TIMESTAMP:
            tr = "timestamps";
            break;
        case wv::HEIGHT:
            tr = "height";
            break;
        case wv::WIDTH:
            tr = "width";
            break;
        case wv::SLOPE:
            tr = "slope";
            break;
        case wv::SLOPE_D:
            tr = "slope_d";
            break;
        case wv::ENERGY:
            tr = "energy";
            break;
        case wv::RAW:
            tr = "raw";
            break;
        case wv::RAW_D:
            tr = "raw_d";
            break;
        case wv::PCA:
            tr = "pca";
            break;
        case wv::PCA_D:
            tr = "pca_d";
            break;
#ifdef USE_GSL
        case wv::SPLINE:
            tr = "spline";
            break;
        case wv::SPLINE_D:
            tr = "spline_d";
            break;
        case wv::FOURIER:
            tr = "fourier";
            break;
        case wv::FOURIER_D:
            tr = "fourier_d";
            break;
#endif
        case wv::WAVELET:
            tr = "wavelet";
            break;
        case wv::WAVELET_D:
            tr = "wavelet_d";
            break;
        case wv::UNKNOWN_TRANSFORM:
            tr = "unknown";
            break;
    }
    return tr;
}

wv::TRANSFORM wv::TransformEnumVal(const std::string& ts)
{
    if (ts=="user") return wv::USER;
    if (ts=="timestamps") return wv::TIMESTAMP;
    if (ts=="height") return wv::HEIGHT;
    if (ts=="width") return wv::WIDTH;
    if (ts=="slope") return wv::SLOPE;
    if (ts=="slope_d") return wv::SLOPE_D;
    if (ts=="energy") return wv::ENERGY;
    if (ts=="raw") return wv::RAW;
    if (ts=="raw_d") return wv::RAW_D;
    if (ts=="pca") return wv::PCA;
    if (ts=="pca_d") return wv::PCA_D;
#ifdef USE_GSL
    if (ts=="spline") return wv::SPLINE;
    if (ts=="spline_d") return wv::SPLINE_D;
    if (ts=="fourier") return wv::FOURIER;
    if (ts=="fourier_d") return wv::FOURIER_D;
#endif
    if (ts=="wavelet") return wv::WAVELET;
    if (ts=="wavelet_d") return wv::WAVELET_D;
    return wv::UNKNOWN_TRANSFORM;
}

std::string wv::ClusterName(wv::CLUSTER_METHOD cluster_method)
{
    std::string clus;
    switch (cluster_method)
    {
        case wv::MANUAL:
            clus = "Manual";
            break;
        case wv::AUTOCLASS:
            clus = "AutoClass";
            break;
        case wv::KMEANS:
            clus = "KMeans";
            break;
        case wv::SPC:
            clus = "SPC";
            break;
        case wv::KK:
            clus = "KlustaKwik";
            break;
        case wv::UNKNOWN_METHOD:
            clus = "Unknown";
    }
    return clus;
}
wv::CLUSTER_METHOD wv::ClusterEnumVal(const std::string& cs)
{
    if (cs=="Manual") return wv::MANUAL;
    if (cs=="AutoClass") return wv::AUTOCLASS;
    if (cs=="KMeans") return wv::KMEANS;
    if (cs=="SPC") return wv::SPC;
    if (cs=="KlustaKwik") return wv::KK;
    return wv::UNKNOWN_METHOD;
}

std::string wv::VersionString()
{
    char cvs[256];
    sprintf(cvs, "%d.%d.%d", wv::VERSION_MAJOR, wv::VERSION_MINOR, wv::VERSION_BUILD);
    return std::string(cvs);
}
