#ifndef WV_H
#define WV_H

#include "stl_headers.h"

namespace wv //Can't call this 'ws' because it conflicts with QTextStream& ws(QTextStream&), apparently
{
    typedef int TRANSFORM_ID; //This is 0 for all canned transforms (incl. raw), counts up for user transforms
    typedef int CLUSTER_ID; //The official 'count' and updating of ids happens in ClusMethodBase

    enum TRANSFORM
    {
        UNKNOWN_TRANSFORM = -1,
        USER, //Created from vectors from other transforms
        HEIGHT,
        WIDTH,
        SLOPE,
        SLOPE_D,
        ENERGY,
        TIMESTAMP, //Up to here are not true transforms, but can populate an axis
        RAW,
        RAW_D,
        PCA,
        PCA_D,
#ifdef USE_GSL
        SPLINE,
        SPLINE_D,
        FOURIER,
        FOURIER_D,
#endif
        WAVELET,
        WAVELET_D
    };

    enum CLUSTER_METHOD
    {
        UNKNOWN_METHOD = -1,
        MANUAL,
        AUTOCLASS,
        KMEANS,
        SPC,  //Superparamagnetic clustering
        KK      //KlustaKwik
    };

    extern const CLUSTER_ID DEFAULT_CID; // ****This value is  used to reference the
        //waveforms currently in WaveSpaceGui, see ClusterMediator
    extern const CLUSTER_ID NO_CLUSTER_ID;

    extern const TRANSFORM_ID DEFAULT_TID; //Since transforms aren't tied to a particular analysis,
        //one ID for them all_ID;
    extern const TRANSFORM_ID NO_TRANSFORM_ID;

/*    struct TRANSFORM_ID
    {
        explicit TRANSFORM_ID(int id_ = DEFAULT_TID) : id(id_) {}
        TRANSFORM_ID& operator++() { ++id; return *this; }
        int id;
    };*/

    extern const int DEFAULT_CLUSTER; //Used in e.g. ManualTO, when clustering is done 'in house'
    extern const int ZERO_CLUSTER; //Used *solely* for manually selected clusters that have become unselected.
    extern const int NULL_CLUSTER;
    extern const int ACCEPT_CLUSTER;
    extern const int REJECT_CLUSTER;

    extern const int VERSION_MAJOR;
    extern const int VERSION_MINOR;
    extern const int VERSION_BUILD;
    const std::string VERSION_SUFFIX();

    std::string TransformName(TRANSFORM transform);
    TRANSFORM TransformEnumVal(const std::string& ts);

    std::string ClusterName(wv::CLUSTER_METHOD cluster_method);
    CLUSTER_METHOD ClusterEnumVal(const std::string& cs);

    std::string VersionString();

}

#endif // WV_H
