#include "profilegui.h"
#include "ui_profilegui.h"
#include "../file/output/profileout.h"

ProfileGui::ProfileGui(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ProfileGui), _initEntryComplete(false)
{
    ui->setupUi(this);

    std::string profiles = FileBase::UsersDir() + "/profiles.txt";
    QFile pfiles(profiles.c_str());
    pfiles.open(QFile::ReadOnly);
    while (!pfiles.atEnd())
    {
        std::string profile = pfiles.readLine().data();
        boost::algorithm::trim(profile);
        _currentProfiles << profile;
    }

}

ProfileGui::~ProfileGui()
{
    delete ui;
}

void ProfileGui::on_edProfile_editingFinished()
{
    if (ui->edProfile->text()=="") return;
    RetrieveProfile( ui->edProfile->text().toStdString() );
}
void ProfileGui::on_btnContinue_clicked()
{
    RetrieveProfile( ui->edProfile->text().toStdString() );
}
void ProfileGui::on_btnNew_clicked()
{
    std::string new_profile = GetNewProfileName();
    if (!new_profile.empty())
        MakeProfile(new_profile);
}

void ProfileGui::closeEvent(QCloseEvent*)
{
//    qDebug() << e->type();
//    exit(EXIT_SUCCESS);
}

void ProfileGui::RetrieveProfile(const std::string& profile)
{
    if (_initEntryComplete) return; _initEntryComplete = true;
    try
    {
        if (profile == "")
            MakeTempProfile();
        else if ( std::find(_currentProfiles.begin(), _currentProfiles.end(), profile)
                  != _currentProfiles.end() )
        {
            emit CurrentProfile(profile);
            close();
        }
        else
        {
            QMessageBox msg_box;
            msg_box.setText("This profile does not exist.");
            msg_box.setInformativeText("Would you like to create a new profile?");
            msg_box.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
            msg_box.setDefaultButton(QMessageBox::Ok);
            int choice = msg_box.exec();

            switch (choice)
            {
                case QMessageBox::Ok:
                    MakeProfile(profile);
                    break;
                case QMessageBox::Cancel:
                    MakeTempProfile();
                    break;
                default:
                    break;
            }
        }
    }
    catch (std::exception& e)
    {
        Log("Profile \"" + profile + "\": " + e.what());
    }
}
std::string ProfileGui::GetNewProfileName()
{
    bool ok;
    std::string new_profile = QInputDialog::getText(this,
                                      "New Profile",
                                      "Enter name: ",
                                      QLineEdit::Normal,
                                      "",
                                      &ok).toStdString();
    if (!ok) return "";
    boost::algorithm::trim(new_profile);
    return new_profile;
}
void ProfileGui::MakeProfile(const std::string& profile)
{
    try
    {
        std::string users_dir = FileBase::UsersDir();
        std::string user_dir = users_dir + "/" + profile;
#ifndef Q_OS_UNIX
        std::string make_dir = "md",
                sep = "&";
#else
        std::string make_dir = "mkdir",
                sep = ";";
#endif

        std::string cmd1 = make_dir + " " + user_dir + " " + sep + " ";
        std::string cmd2 = make_dir + " " + user_dir + "/temp " + sep + " ";

        std::string local_data_dir = user_dir + "/data";
        std::string local_wsdata_dir = user_dir + "/wsdata";
        std::string cmd3 = make_dir + " " + local_data_dir + " " + sep + " ";
        std::string cmd4 = make_dir + " " + local_wsdata_dir;
            //The user need not actually choose these, but they're there if he/she wants.

        std::string cmd_all = cmd1 + cmd2 + cmd3 + cmd4;
#ifndef Q_OS_UNIX
        std::replace(cmd_all.begin(), cmd_all.end(), '/', '\\');
#endif
        system( cmd_all.c_str() );

        std::string data_dir, wsdata_dir;
        if (profile != "temp" + std::to_string( QApplication::applicationPid() ))
        {
            if ( std::find(_currentProfiles.begin(), _currentProfiles.end(), profile)
                 != _currentProfiles.end() )
            {
                QErrorMessage err_dlg;
                err_dlg.setWindowModality(Qt::ApplicationModal);
                err_dlg.showMessage("Profile name already taken, please choose another");
                connect(&err_dlg, SIGNAL(finished(int)), &err_dlg, SLOT(close()));
                err_dlg.exec();
                std::string new_profile = GetNewProfileName();
                MakeProfile(new_profile);
                return;
            }

            data_dir = QFileDialog::getExistingDirectory(this,
                        "Choose your default data directory").toStdString();
            wsdata_dir = QFileDialog::getExistingDirectory(this,
                         "Choose the WaveSorter output directory").toStdString();

            std::string cmd12 = cmd1 + cmd2;
#ifndef Q_OS_UNIX
    std::replace(cmd12.begin(), cmd12.end(), '/', '\\');
#endif
            system( cmd12.c_str() );

            //Now add the new profile name to profiles.txt
            _currentProfiles << profile;
            QFile pf( (users_dir + "/profiles.txt").c_str() );
            pf.open(QFile::WriteOnly);
            pf.seek(0);
            std::string profiles = boost::algorithm::join(_currentProfiles, "\n");
            pf.write(profiles.c_str());
            pf.close();
        }

    //Now create the user's profile file

//    std::string cmd5 = "echo \"Data Directory:\\t" + data_dir
//            + "\\nWSData Directory:\\t" + wsdata_dir + "\\n\" > " + user_dir + "/profile.txt;";

        session_profile.AddParList("Session", SessionPars().DefaultParList());
        session_profile.AddPar("Session", Param("Data Directory", data_dir));
        session_profile.AddPar("Session", Param("WS Data Directory", wsdata_dir));
        FileBase::SetUserName(profile); //Necessary to do this here in advance of ProfileOut::Save()
        ProfileOut profile_out;
        profile_out.Save();
        profile_out.Close();  //Have to close here because it takes time to close the file pointer, and
        //and WaveSorter will try to open it in MainWindow::ReceiveProfile before it is!
//    system(cmd5.toAscii());

        emit CurrentProfile(profile);
        close();
    }
    catch (std::exception& e)
    {
        Log("Caught in ProfileGui::MakeProfile, Profile \"" + profile + "\": " + e.what());
    }
}

void ProfileGui::MakeTempProfile()
{
    ui->edProfile->setText("");
    long long int pid = QApplication::applicationPid();
    std::string profile = "temp" + std::to_string(pid);
    MakeProfile(profile);
    Log("Using default profile.  Parameter changes will not be saved.");
}
