#include "aboutgui.h"
#include "ui_aboutgui.h"

#include "../../globals/globals.h"

AboutGui::AboutGui(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AboutGui)
{
    ui->setupUi(this);

    char cname[256];
    sprintf(cname, "WaveSorter, version %d.%d.%d%s.",
            wv::VERSION_MAJOR, wv::VERSION_MINOR, wv::VERSION_BUILD, wv::VERSION_SUFFIX().c_str());
    std::string name(cname);

    std::string about_str = name + "\n\n" \

        "by Matt Phillips (mattphillipsphd@gmail.com).\n\n" \

        "WaveSorter is a tool for sorting waveforms.  It is intended for " \
        "sorting of neural action potentials but it makes very few assumptions about " \
        "the structure of the data.  So it could in principle be used " \
        "with any kind of data, especially time-series data.\n\n" \
        "WaveSorter, in particular the GUI, is built with the Qt library.";

        about_str +=
#ifdef USE_GSL
        "  WaveSorter uses the GNU Scientific library v. 1.15 for all transformations " \
        "it implements."
#endif
        "For clustering it provides an interface to a manual sorting procedure, " \
        "as well as interfaces to:\n" \
        "\t1) David Arthur's k-means++ program (http://www.stanford.edu/~darthur/kmpp.zip, downloaded Dec. 2011). \n" \
        "\t2) Ken Harris' KlustaKwik clustering program, v. 2.0.1 (http://klustakwik.sourceforge.net/).\n" \
        "\t3) AutoClass C, hosted at http://ti.arc.nasa.gov/tech/rse/synthesis-projects-applications/autoclass/autoclass-c/ ." \
        " See also P. Cheeseman, J. Stutz, \"Bayesian Classification (AutoClass): Theory and Results\", in " \
        "Advances in Knowledge Discovery and Data Mining, Usama M. Fayyad, et al., Eds. AAAI Press/MIT Press, 1996. \n"
#ifdef USE_GSL
        "\t4) Eytan Domany's implementation (v. 2.1) of his Super-Paramagnetic clustering algorithm " \
        "(see Blatt M, Wiseman S, and Domany, E., Physical Review Letters 1996). \n"
#endif

        "\nThese programs have been modified slightly so as to be compatible with the internal " \
        "file format used by WaveSorter, but in no case has the algorithm been changed.\n\n";

#ifdef USE_GSL
      about_str += "If you use WaveSorter in published work, please cite it as: \n\n" \
      "Phillips, M. H.  WaveSorter: A new free- and open-source tool for the offline analysis and " \
      "classification of neural waveforms. Program No. 508.12. 2012 Neuroscience Meeting Planner. " \
      "New Orleans, LA: Society for Neuroscience, 2012. Online.\n\n";
#endif

#ifdef USE_GSL
        about_str += "This version of WaveSorter is licensed " \
        "under the GPL v. 3.1. Copyright Matt Phillips, 2012.";
#else
        about_str += "Copyright Matt Phillips, 2012.  This version of WaveSorter is not free software; " \
                "you are not allowed to distribute WaveSorter without permission.";
#endif
        about_str += "The copyrights of the " \
        "external clustering programs belong to their respective owners, and are licensed separately; " \
        "see information in their corresponding file folders for details.\n\n";

    ui->txtAbout->setText(about_str.c_str());
}

AboutGui::~AboutGui()
{
    delete ui;
}

void AboutGui::SetText(const std::string& text)
{
    ui->txtAbout->setText(text.c_str());
}
