#ifndef SCANWAVEWIDGET_H
#define SCANWAVEWIDGET_H

#include "wavewidget.h"
#include "../../../memrep/drawing/polyline.h"
#include "../../../memrep/specs/clusterspec.h"

class ScanWaveWidget : public WaveWidget
{
    Q_OBJECT

    public:
        ScanWaveWidget(QWidget* parent);

        void InitGraphics(CluSpecPtr clu_spec);
        virtual void SetWaves(wv::CLUSTER_ID cid, wv::TRANSFORM trans, wv::TRANSFORM_ID tid,
                                  const std::vector<QColor>* all_colors, bool reset_samples = false) override;

        virtual void SetSamplesBegin(int samples_begin) override { _cluSpec->GetWSGSpec().SetSamplesBegin(samples_begin); }
        virtual void SetSamplesEnd(int samples_end) override { _cluSpec->GetWSGSpec().SetSamplesEnd(samples_end); }
        void SetScanline(int sample);

        virtual int SamplesBegin() const override { return _cluSpec->GetWSGSpec().SamplesBegin(); }
        virtual int SamplesEnd() const override { return _cluSpec->GetWSGSpec().SamplesEnd(); }

    signals:
        void ToggleWaveform(int scrub_index);

    protected:
        virtual void paintEvent(QPaintEvent* event) override;
        virtual void mousePressEvent (QMouseEvent* event) override;
        virtual void mouseMoveEvent(QMouseEvent *) override; //NB by default these occur only when the mouse button is pressed.
        virtual void mouseReleaseEvent (QMouseEvent*) override;
        virtual void wheelEvent(QWheelEvent* event) override;

    private:
        int _sample; //Which sample is selected by the sliding cursor

        CluSpecPtr _cluSpec;
        const std::list<PLSpec>* _polylines;
};

#endif // SCANWAVEWIDGET_H
