#ifndef BIELLIPSE_H
#define BIELLIPSE_H

#include "circle.h"
#ifdef USE_GSL
#include "../../analysis/gsl_utils/mgsl.h"
#endif
//The idea of a biellipse is that clusters along any dimension often aren't distributed normally
//but rather take on a log gaussian shape.  Quad ellipses calculate the SD along each dimension, for
//points greater and points less than the center, separately, for a total of 4.  So you can get kind
//of a lopsided ellipse, but in just those cases where it's probably what you want.

//121025 Well, it's just a regular ellipse for now.
class BiEllipse : public Circle
{
    public:
        static const wv::float_t DEFAULT_ELLIPSE_SD;

        BiEllipse();
        BiEllipse(ID id);
        BiEllipse(const BiEllipse& bi_ellipse);
        BiEllipse& operator=(const BiEllipse& bi_ellipse);
        virtual DrawLineBase* Clone() override { return new BiEllipse(*this); }
        virtual ~BiEllipse() override;

        void InitEllipse(const wv_vector* x, const wv_vector* y, const Circle* circle);

        void SetSD(wv::float_t sd);

        wv::float_t SD() const { return _sd; }

    private:
        static const wv::float_t GRAIN;

        void Copy(const BiEllipse& bi_ellipse);
        void UpdateBiEllipse(); //Assumes _sd has already been set

        wv::float_t _centerX, _centerY;
        boost::dynamic_bitset<> _inDistribution; //Contained within the initial circle
        wv_matrix* _rotMatrix; //Angle of rotation induced by the PCA transformation
        wv::float_t _sd; //All that really matters is _isContained and DrawLineBase::_points, but
            //it's good to have this for the parameter file.
        wv::float_t _xCenter, _yCenter;
        wv::float_t _xSD, _ySD; //The value of 1 SD in either dimension.
};

struct BiEllipseSpec : public CircleSpec
{
    BiEllipseSpec(wv::TRANSFORM tr_x, wv::TRANSFORM tr_y, int s_x, int s_y, BiEllipse* be)
        : CircleSpec(tr_x, tr_y, s_x, s_y, be)
    {
    }
    BiEllipseSpec() {}
    virtual ~BiEllipseSpec() override {}

    virtual BiEllipse* GetDrawLine() override
    {
        return static_cast<BiEllipse*>( DrawLineBaseSpec::GetDrawLine() );
    }
    virtual const BiEllipse* GetDrawLine() const override
    {
        return static_cast<const BiEllipse*>( DrawLineBaseSpec::GetDrawLine() );
    }
};

#endif // BIELLIPSE_H
