#ifndef DRAWLINEBASE_H
#define DRAWLINEBASE_H

#include "../../globals/globals.h"
#include "../../memrep/specs/scalespec.h"

class DrawLineBase
{
    public:
        typedef int ID;
        static const int DRAWLINE_MAXLEN;

        DrawLineBase(ID id);
        DrawLineBase();
        DrawLineBase(const DrawLineBase& drawline);
        DrawLineBase& operator=(const DrawLineBase& drawline);
        virtual DrawLineBase* Clone() = 0;
        virtual ~DrawLineBase();

        static ID NewDrawLineID();

        virtual void AddPoint(QPointF point);
        void SetIsFinished(bool is_finished) { _isFinished = is_finished; }

        ID GetID() const { return _id; }
        int Length() const { return _length; }
        wv::float_t X(int i) const { return _points[i].x(); }
        wv::float_t Y(int i) const { return _points[i].y(); }
        QPointF Point(int i) const;
        const QPointF* Points() const { return _points; }
        bool IsFinished() const { return _isFinished; }

    private:
        static int _drawLineIDCt;

        void Copy(const DrawLineBase& drawline);
        ID _id;
        bool _isFinished; //Mainly for circles

    protected: //A little faster and more convenient, for now
        int _length;
        QPointF* _points; // ***Native data values***, e.g. timestamps and voltages
};

struct DrawLineBaseSpec
{
    DrawLineBaseSpec(wv::TRANSFORM tr_x, wv::TRANSFORM tr_y, int s_x, int s_y, DrawLineBase* drawline_)
        : draw_line(drawline_), transform_x(tr_x), transform_y(tr_y), sample_x(s_x), sample_y(s_y),
          time_begin(0), time_end(-1)
    {
    }
    DrawLineBaseSpec() : draw_line(nullptr),
        transform_x(wv::UNKNOWN_TRANSFORM), transform_y(wv::UNKNOWN_TRANSFORM),
        sample_x(-1), sample_y(-1), time_begin(0), time_end(-1)
    {
    }
    DrawLineBaseSpec(const DrawLineBaseSpec& dlbs);
    virtual ~DrawLineBaseSpec() { if (draw_line) delete draw_line; }

    virtual DrawLineBase* GetDrawLine() = 0;
    virtual const DrawLineBase* GetDrawLine() const = 0;

    DrawLineBase* draw_line;
    wv::TRANSFORM transform_x, transform_y;
    int sample_x, sample_y;
    int time_begin, time_end; //NB these are not initialized, have to be set
    std::string Name() const;
};
typedef DrawLineBaseSpec DLBSpec;
    //NB, NO tid, user transforms not allowed with Manual sorting.

#endif // DRAWLINEBASE_H
