#include "histwgtto.h"

HistWgtTO::HistWgtTO(int thd_num, uchar* pixels, const HistWidget* hist_widget, int jstart, int jend, double max_val) : WorkerTO(thd_num),
      _pixels(pixels), _histWidget(hist_widget), _scaleSpec(hist_widget->_scaleSpec), _histAccept(hist_widget->_histAccept), _histUnaccept(hist_widget->_histUnaccept),
      _histWidth(hist_widget->width()), _jStart(jstart), _jEnd(jend), _maxVal(max_val)
{
}

void HistWgtTO::Start() //slot
{
    for (int i=0; i<_histWidth; ++i)
        for (int j=_jStart; j<_jEnd; ++j)
        {
            int idx = (j*_histWidth + i) * 4;

            QPointF pos = _histWidget->Zoom(i, j, true);
            double xzoom = _scaleSpec.PixToValX( pos.x() ),
                    yzoom = _scaleSpec.PixToValY( pos.y() );

            size_t ibin, jbin;
            gsl_histogram2d_find(_histAccept, xzoom, yzoom, &ibin, &jbin);
            int green = (255.0 * gsl_histogram2d_get(_histAccept, ibin, jbin)
                                  / _maxVal + 0.5);
            int white = qMin(255.0, 255.0 * gsl_histogram2d_get(_histUnaccept, ibin, jbin)
                                  / _maxVal + 0.5);

            _pixels[idx+3] = 0xff;
            _pixels[idx+2] = (uchar)white;
            _pixels[idx+1] = (uchar)qMin(255, white+green);
            _pixels[idx] = (uchar)white;
                //Don't understand why but reverse order is required
        }

    WorkFinished();
}
