#include "paramsbase.h"

ParamsBase::ParamsBase(const std::string& name) : _name(name)
{
    if (session_profile.IsParList(_name))
        _activePars = session_profile.ParList(_name);
}

void ParamsBase::RestoreDefaults()
{
    _activePars = _defaultPars;
}

void ParamsBase::SetPar(const std::string& key, const std::string& value)
{
    ParamsIter it = _activePars.begin();
    while (it!=_activePars.end())
    {
        if (it->first == key)
        {
            it->second = value;
            break;
        }
        ++it;
    }
}
void ParamsBase::SetPar(int i, const std::string& value)
{
    _activePars[i].second = value;
}
std::string ParamsBase::Par(const std::string& key) const
{
    ParamsCIter it = _activePars.begin();
    while (it!=_activePars.end())
    {
        if (it->first == key)
            return it->second;
        ++it;
    }
    return Profile::BAD_PARAMETER;
}
std::string ParamsBase::Key(int i) const
{
    return _activePars.at(i).first;
}
std::string ParamsBase::Value(int i) const
{
    return _activePars.at(i).second;
}
void ParamsBase::Insert(const std::string& key, const std::string& value, bool is_internal)
{
    if ( Par(key) == Profile::BAD_PARAMETER )
        _activePars.push_back( Param(key, value) );
        //So, constructor values are only inserted into the *_activePars* list
        //if they're missing from the profile.
    _defaultPars.push_back( Param(key, value) );

    _isInternal[key] = is_internal;
}
