#ifndef PARAMETERS_H
#define PARAMETERS_H

#include "../../globals/globals.h"
#include "../../globals/log.h"
#include "profile.h"

typedef ParamVec::const_iterator ParamsCIter;
typedef ParamVec::iterator ParamsIter;

class ParamsBase
{
    friend class Profile;
    public:
        ParamsBase(const std::string& name);
        virtual ~ParamsBase() {}

        void RestoreDefaults();

        void SetPar(const std::string& key, const std::string& value);
        void SetPar(int i, const std::string& value);

        const ParamVec& DefaultParList() const { return _defaultPars; }
        const std::string& Name() const { return _name; }
        const ParamVec& ParList() const { return _activePars; }
        std::string Par(const std::string& key) const;
        int NumPars() const { return _activePars.size(); }
        std::string Key(int i) const;
        std::string Value(int i) const;
        bool IsInternal(const std::string& key) const { return _isInternal.at(key); }

    protected:
        void Insert(const std::string& key, const std::string& value, bool is_internal = false);

    private:
        std::string _name;
        ParamVec _activePars, _defaultPars; //Want to preserve the order that
            //they're specified in clustering algorithms, so don't use a map, which
            //alphabetizes them
        std::map<std::string, bool> _isInternal; //If true, this won't be shown in the parameter gui.
};

typedef std::shared_ptr<ParamsBase> ParamsPtr;
typedef std::shared_ptr<const ParamsBase> ParamsCPtr;

#endif // PARAMETERS_H
