#include "profile.h"

const std::string Profile::BAD_PARAMETER = "#@#";
Profile::Profile()
{
}

void Profile::AddParList(const std::string& list_name, const ParamVec& pars)
{
    _paramLists[list_name] = pars;
}
void Profile::AddPar(const std::string& list_name, const Param& par)
{
    std::string key = par.first;
    if ( Par(list_name, key) == BAD_PARAMETER )
        Log("Warning: parameter " + key + " for list " + list_name + " not found.");
    else
        SetPar(list_name, key, par.second);
}

void Profile::Write(FILE* fp, const std::string& list_name) const
{
    char session[256];
    strcpy(session, std::string("@" + list_name + "\n").c_str());
    int br = fprintf(fp, session);
    if (br==0)
        throw WIOException("Profile::Write", "code: " + std::to_string(errno));

    const ParamVec& param_list = _paramLists.at(list_name);
    for (auto itp : param_list)
    {
        char par[256];
        strcpy(par, std::string("\t" + itp.first + ":\t" + itp.second + "\n").c_str());
        fprintf(fp, par);
    }
}

void Profile::SetPar(const std::string& list_name, const std::string& key, const std::string& value)
{
    ParamVec& par_list = _paramLists[list_name];
    size_t ct = 0;
    for (auto& it : par_list)
    {
        if (key == it.first)
        {
            it.second = value;
            break;
        }
        ++ct;
    }
    if (ct==par_list.size())
        throw BadParameter("Profile::SetPar", key);
}

bool Profile::IsParList(const std::string& name) const
{
    return _paramLists.find(name) != _paramLists.end();
}

std::string Profile::Par(const std::string& list_name, const std::string& key) const
{
    if (_paramLists.find(list_name) == _paramLists.end())
        throw BadParameter("Profile::Par", list_name + "; " + key);
    const ParamVec& par_list = _paramLists.at(list_name);
    for (auto it : par_list)
        if (key == it.first)
            return it.second;
    return BAD_PARAMETER;
}

Profile session_profile;
