#ifndef RESULTSPEC_H
#define RESULTSPEC_H

#include "../../globals/globals.h"
#include "../classification.h"

//This holds all the information required to determine a specific clustering.  It is best
//to *not* simply stick this all in classification; that should rather be const after
//creation.  The other members of this struct can be modified by the user.
// ***Only*** setter/getter, add/remove methods in this and all specs classes.
//Private methods used only here (not by friends) are an exception.

//Note that scrubbed records are *not* taken into account in this data set, have to bring in WaveData
//for that.

//*** There is a substantial amount of redundancy between _acceptedClus, _vAcceptedClus, and _isAccepted,
//but seems best for speed--have to be very careful with these.
//  ###In particular _acceptedClus can be tossed, only using the number of accepted clusters, not which ones
//they are.
class ResultSpec
{
    public:
        ResultSpec();
        ResultSpec(ClassificationCPtr cls, int level = 0);
        ResultSpec(const ResultSpec& result_spec);
        ResultSpec& operator=(const ResultSpec& result_spec);
        ~ResultSpec();

        void AddClusters(const std::vector<int>& clusters); //Replaces current cluster set with these

        void SetClusAccepted(wv_vector_int* clus_accepted);
        void SetLevel(int level);
        void SetStats(int level, const std::vector<wv::float_t>& stats);

        const boost::dynamic_bitset<>& Accepted() const { return _isAccepted.at(_level); }
        const wv_vector_int* ClusAccepted() const; //This returns wv::ACCEPT_CLUSTER for all clusters
            //of a clustering that have been accepted, and the cluster # otherwise
        const ClassificationCPtr GetClassification() const { return _cls; }
        int Level() const { return _level; } //The *index*
        wv::float_t LevelVal() const; //The actual value, which for e.g. SPC is different than the index
        const std::vector<wv::float_t>& Stats(int level) const;

    private:
        void CalcAccepted(); //Call this every time _vAcceptedClus changes.  Since bitsets are
            //so compact, we calculate and store for every level.
        void Copy(const ResultSpec& result_spec);

        std::map<int, std::unordered_set<int> > _acceptedClus;
            //Which clusters, among those available at level, are selected.
        ClassificationCPtr _cls;
        std::map<int, boost::dynamic_bitset<> > _isAccepted; //yes or no for each level
        int _level; //An index; not e.g. the actual temperature
        std::map<int, std::vector<wv::float_t> > _stats; //A number indicating the quality of each cluster
        wv_vector_int* _vAcceptedClus; //For every waveform, the cluster id (wv::ACCEPT_CLUSTER for all
            //accepted clusters)
};

#endif // RESULTSPEC_H
