#include "scalespec.h"

#include "../../globals/globals.h"

ScaleSpec::ScaleSpec(wv::float_t x1_val, wv::float_t x2_val, wv::float_t y1_val, wv::float_t y2_val,
          wv::float_t x1_px, wv::float_t x2_px, wv::float_t y1_px, wv::float_t y2_px)
    : _xMinVal(x1_val), _xMaxVal(x2_val), _yMinVal(y1_val), _yMaxVal(y2_val),
    _xMinPix(x1_px), _xMaxPix(x2_px), _yMinPix(y1_px), _yMaxPix(y2_px)
{
}
ScaleSpec::ScaleSpec() : _xMinVal(0.0), _xMaxVal(0.0), _yMinVal(0.0), _yMaxVal(0.0),
    _xMinPix(0.0), _xMaxPix(0.0), _yMinPix(0.0), _yMaxPix(0.0)
{
}

void ScaleSpec::SetValuesX(wv::float_t x_min_val, wv::float_t x_max_val)
{
    _xMinVal = x_min_val;
    _xMaxVal = x_max_val;
}
void ScaleSpec::SetValuesY(wv::float_t y_min_val, wv::float_t y_max_val)
{
    _yMinVal = y_min_val;
    _yMaxVal = y_max_val;
}
void ScaleSpec::SetValuesPix(wv::float_t x_min_px, wv::float_t x_max_px, wv::float_t y_min_px, wv::float_t y_max_px)
{
    _xMinPix = x_min_px;
    _xMaxPix = x_max_px;
    _yMinPix = y_min_px;
    _yMaxPix = y_max_px;
}


QPointF ScaleSpec::PixToVal(QPointF pix) const
{
    wv::float_t x = PixToValX(pix.x()),
            y = PixToValY(pix.y());
    return QPointF(x, y);
}
wv::float_t ScaleSpec::PixToValX(wv::float_t xpix) const
{
    return (_xMaxVal-_xMinVal) * xpix / (_xMaxPix-_xMinPix) + _xMinVal;
}
void ScaleSpec::PixToValXVec(wv_vector* xpix_new, const wv_vector* xpix_old) const
{
    if (xpix_old) wv_vector_memcpy(xpix_new, xpix_old);
    wv_vector_scale(xpix_new, (_xMaxVal-_xMinVal)/(_xMaxPix-_xMinPix));
    wv_vector_add_constant(xpix_new, _xMinVal);
}
wv::float_t ScaleSpec::PixToValY(wv::float_t ypix) const
{
    return (_yMaxVal-_yMinVal) * (1.0 - ypix / (_yMaxPix-_yMinPix)) + _yMinVal;
}
void ScaleSpec::PixToValYVec(wv_vector* ypix_new, const wv_vector* ypix_old) const
{
    if (ypix_old) wv_vector_memcpy(ypix_new, ypix_old);
    wv_vector_scale(ypix_new, 1.0/(_yMaxPix-_yMinPix));
    wv_vector_add_constant(ypix_new, -1.0);
    wv_vector_scale(ypix_new, -1.0*(_yMaxVal-_yMinVal));
    wv_vector_add_constant(ypix_new, _yMinVal);
}

QPointF ScaleSpec::ValToPix(QPointF val) const
{
    wv::float_t x = ValToPixX(val.x()),
            y = ValToPixY(val.y());
    return QPointF(x, y);
}
wv::float_t ScaleSpec::ValToPixX(wv::float_t x) const
{
    return (_xMaxPix-_xMinPix) * (x-_xMinVal) / (_xMaxVal-_xMinVal);
}
void ScaleSpec::ValToPixXVec(wv_vector* xpix_new, const wv_vector* xpix_old) const
{
    if (xpix_old) wv_vector_memcpy(xpix_new, xpix_old);
    wv_vector_add_constant(xpix_new, -_xMinVal);
    wv_vector_scale(xpix_new, (_xMaxPix-_xMinPix)/(_xMaxVal-_xMinVal));
}
wv::float_t ScaleSpec::ValToPixY(wv::float_t y) const
{
    return (_yMaxPix-_yMinPix) * (1.0 - (y-_yMinVal) / (_yMaxVal-_yMinVal));
}
void ScaleSpec::ValToPixYVec(wv_vector* ypix_new, const wv_vector* ypix_old) const
{
    if (ypix_old) wv_vector_memcpy(ypix_new, ypix_old);
    wv_vector_add_constant(ypix_new, -_yMinVal);
    wv_vector_scale(ypix_new, -1.0/(_yMaxVal-_yMinVal));
    wv_vector_add_constant(ypix_new, 1.0);
    wv_vector_scale(ypix_new, (_yMaxPix-_yMinPix));
}
