#include "usertransformspec.h"

int UserTransformSpec::_userTransformId = wv::DEFAULT_TID + 1;
void UserTransformSpec::AddVector(wv_vector_const_view view, wv::TRANSFORM transform, int sample)
{
    std::pair<wv::TRANSFORM, int> user_spec(transform, sample);
    if ( std::find(_signature.begin(), _signature.end(), user_spec) != _signature.end() )
    {
        Log("WARNING: This vector is already present in the User transform, not adding.");
        return;
    }

    int num_samples = (_waves) ? _waves->size2 : 0,
            num_records = view.vector.size;
    if (!_waves)
        _waves = MatPtr( wv_matrix_alloc(num_records, 1), wv_matrix_free );
    else
    {
        wv_matrix* new_waves = wv_matrix_alloc(num_records, num_samples + 1);
        new_waves->size2 = num_samples; //We 'trick' memcpy into thinking the matrices are the same
            //size; the extra column is of course for the vector to be added.
        wv_matrix_memcpy(new_waves, Waves().get());
        _waves.reset(new_waves);
        _waves->size2 = num_samples + 1;
    }
    wv_vector_view col = wv_matrix_column(_waves.get(), num_samples);
    wv_vector_memcpy(&col.vector, &view.vector);

    wv::float_t min, max;
    wv_vector_minmax(&col.vector, &min, &max);
    wv::float_t range = max - min;
    wv_vector_add_constant(&col.vector, -min);
    wv_vector_scale(&col.vector, 1000.0/range);

    _signature.push_back( std::pair<wv::TRANSFORM, int>(transform, sample) );
}
void UserTransformSpec::RemoveVector(int col)
{
    if (!_waves) return;
    int num_records = _waves->size1,
            num_samples = _waves->size2;
    if (num_samples == 1)
        _waves.reset();
    else
    {
        wv_matrix* new_waves = wv_matrix_alloc(num_records, num_samples - 1);
        for (int j=0, ct=0; j<num_samples; ++j)
        {
            if (j!=col)
            {
                wv_vector_view view_old = wv_matrix_column(_waves.get(), j),
                        view_new = wv_matrix_column(new_waves, ct++);
                wv_vector_memcpy(&view_new.vector, &view_old.vector);
            }
        }
        _waves.reset(new_waves);
    }

    _signature.erase( _signature.begin()+col );
}
