#ifndef WAVETEMPLATE_H
#define WAVETEMPLATE_H

#include "../globals/globals.h"

//In addition to the template (average waveform) corresponding to each cluster,
//this class also holds the z-transformed distance of every waveform from that cluster.

//These are built and stored in the corresponding *WaveWidgets*.

class WaveTemplate
{
    public:
        WaveTemplate(MatCPtr waves, const boost::dynamic_bitset<>& window,
                     const std::vector<int>& selected, const wv_vector_int* vcluster);
        WaveTemplate(const WaveTemplate& wave_template);
        WaveTemplate& operator=(const WaveTemplate& wave_template);
        ~WaveTemplate();

        const std::map<int, wv_vector*>& Templates() const { return _templates; }
        const std::map<int, wv_vector*>& TemplateSDs() const { return _templateSDs; }

    private:
        int ClusterCount(int c, const boost::dynamic_bitset<>& window);
        void Copy(const WaveTemplate& wave_template);
        void DeleteTemplates();

        const wv_vector_int* _vCluster;
        std::map<int, wv_vector*> _templates, //A map from cluster # to template
                            _templateSDs; //The corresponding standard deviation, defined at each sample
};

#endif // WAVETEMPLATE_H
